@@ -695,7 +695,7 @@ connect_dialog_response_cb (NautilusConnectServerDialog *dialog,
 	case GTK_RESPONSE_HELP :
 		error = NULL;
 		gtk_show_uri (gtk_window_get_screen (GTK_WINDOW (dialog)),
-			      "ghelp:gnome-help#nautilus-connect",
+			      "help:gnome-help/nautilus-connect",
 			      gtk_get_current_event_time (), &error);
 		if (error) {
 			eel_show_error_dialog (_("There was an error displaying help."), error->message,
@@ -1243,6 +1243,12 @@ nautilus_connect_server_dialog_fill_details_async (NautilusConnectServerDialog *
 			g_mount_operation_set_password (G_MOUNT_OPERATION (operation),
 							str);
 			set_flags ^= G_ASK_PASSWORD_NEED_PASSWORD;
+			
+			if (flags & G_ASK_PASSWORD_SAVING_SUPPORTED &&
+			    gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (self->details->remember_checkbox))) {
+				g_mount_operation_set_password_save (G_MOUNT_OPERATION (operation),
+								     G_PASSWORD_SAVE_PERMANENTLY);
+			}
 
 			self->details->last_password_set = TRUE;
 		}
